namespace G4 {

    public class Switcher : Gtk.Widget {
        static construct {
            set_layout_manager_type (typeof (Gtk.BoxLayout));
        }

        private Gtk.Stack _stack = (Gtk.Stack) null;

        public Switcher (bool narrow, uint spacing = 4) {
            var layout = get_layout_manager () as Gtk.BoxLayout;
            layout?.set_homogeneous (true);
            layout?.set_spacing (spacing);
            add_css_class (narrow ? "narrow" : "wide");
        }

        public Gtk.Stack stack {
            get {
                return _stack;
            }
            set {
                _stack = value;
                _stack.bind_property ("visible-child-name", this, "visible-child-name");
                update_buttons ();
            }
        }

        public string visible_child_name {
            set {
                for (var child = get_first_child (); child != null; child = child?.get_next_sibling ()) {
                    var button = (Gtk.ToggleButton) child;
                    button.active = button.name == value;
                }
            }
        }

        public void update_buttons () {
            var pages = _stack.pages;
            var n_items = pages.get_n_items ();
            for (var child = get_last_child (); child != null; child = child?.get_prev_sibling ()) {
                if (_stack.get_child_by_name (((!)child).name) == null)
                    ((!)child).unparent ();
            }
            var visible_child = _stack.visible_child;
            for (var i = 0; i < n_items; i++) {
                var page = (Gtk.StackPage) pages.get_item (i);
                if (find_child_by_name (page.name) == null) {
                    var button = new Gtk.ToggleButton ();
                    button.active = page.child == visible_child;
                    button.hexpand = true;
                    button.icon_name = page.icon_name;
                    button.name = page.name;
                    button.tooltip_text = page.title;
                    button.width_request = 48;
                    button.add_css_class ("flat");
                    button.toggled.connect (() => {
                        if (button.active && _stack.visible_child_name != button.name)
                            _stack.set_visible_child_name (button.name);
                        else if (!button.active && _stack.visible_child_name == button.name)
                            run_idle_once (() => button.active = true);
                    });
                    button.insert_before (this, null);
                }
            }
            queue_resize ();
        }

        private Gtk.Widget? find_child_by_name (string name) {
            for (var child = get_first_child (); child != null; child = child?.get_next_sibling ()) {
                if (((!)child).name == name)
                    return child;
            }
            return null;
        }
    }

    public class NarrowBar : Gtk.Widget {
        private Gtk.Widget? _child = null;
        private int _minimum_width = 0;
        private bool _reveal = true;

        public Gtk.Widget? child {
            get {
                return _child;
            }
            set {
                _child?.unparent ();
                _child = value;
                _child?.set_parent (this);
                queue_allocate ();
            }
        }

        public bool reveal {
            get {
                return _reveal;
            }
            set {
                _reveal = value;
                queue_allocate ();
            }
        }

        public override void measure (Gtk.Orientation orientation, int for_size, out int minimum, out int natural, out int minimum_baseline, out int natural_baseline) {
            if (_child != null) {
                ((!)_child).measure (orientation, for_size, out minimum, out natural, out minimum_baseline, out natural_baseline);
            } else {
                minimum = natural = minimum_baseline = natural_baseline = 0;
            }
            if (orientation == Gtk.Orientation.HORIZONTAL) {
                _minimum_width = minimum;
                minimum = _minimum_width / 2;
            }
        }

        public override void size_allocate (int width, int height, int baseline) {
            var wide = width >= _minimum_width;
            if (_reveal != wide) {
                _reveal = wide;
                notify_property ("reveal");

                if (wide && _child?.parent == null) {
                    _child?.set_parent (this);
                } else if (!wide && _child?.parent != null) {
                    _child?.unparent ();
                }
            }

            if (_child != null) {
                var allocation = Gtk.Allocation ();
                allocation.x = 0;
                allocation.y = 0;
                allocation.width = width;
                allocation.height = height;
                ((!)_child).allocate_size (allocation, baseline);
            }
        }
    }
}
